/*
 *  linux/include/asm-arm/hardware/psionw.h
 *
 *  This file contains the hardware definitions of the PSIONW internal
 *  registers.
 *
 *  Copyright (C) 2001 Tony Lindgren <tony@atomide.com>
 *  Copyright (C) 2001 Yuji Shinokawa <ysh@mob.or.jp>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ************************ NOTE FOR DEVELOPERS ***************************
 *  
 *  Please see also the "Windermere Software Interface Specification"
 *  PDF file. (Series_5mx_specs.pdf)
 *        
 *  This file is not publicly available, but is available for Linux 
 *  developers without a NDA. Thank you Psion!
 *
 *  Please contact the PsiLinux mailing list at:
 *
 *    linux-7110-psion@lists.sourceforge.net 
 *
 *  to get a copy of this file.
 * 
 *  ************************************************************************
 */

#ifndef __ASM_HARDWARE_PSIONW_H
#define __ASM_HARDWARE_PSIONW_H

#define PSIONW_PHYS_BASE	(0x80000000)

#ifndef __ASSEMBLY__
#define psionw_readb(off)	__raw_readb(PSIONW_BASE + (off))
#define psionw_readl(off)	__raw_readl(PSIONW_BASE + (off))
#define psionw_writeb(val,off)	__raw_writeb(val, PSIONW_BASE + (off))
#define psionw_writel(val,off)	__raw_writel(val, PSIONW_BASE + (off))
#endif

/*
 * LCD Controller
 */

#define PSION_HSW	(0x02 << 10)	/* Horiz sync pulse width */
#define PSION_HFP	(0x01 << 16)	/* Horiz front porch */
#define PSION_HBP	(0x01 << 24)	/* Horiz back porch */

#define PSION_VSW	(0x02 << 10)	/* Vert sync pulse width */
#define PSION_VFP	(0x01 << 16)	/* Vert front porch */
#define PSION_VBP	(0x01 << 24)	/* Vert back porch */

#if defined(CONFIG_PSIONW_REVO) || defined(CONFIG_PSIONW_REVOPLUS)
#define PSION_LCD_W	480		/* LCD width */
#define PSION_LCD_H	160		/* LCD heigth */
#define PSION_ACB	(0xc << 8)	/* AC-bias frequency */
#else
#define PSION_LCD_W	640		/* LCD width */
#define PSION_LCD_H	240		/* LCD height */
#define PSION_ACB	(0xb << 8)	/* AC-bias frequency */
#endif

#define PSION_LCD_OFF	0xd000		/* LCD memory start offset */
#define LCD_PAL_SIZE	(0x20)		/* LCD palette size */
#define LCD_BUF_SIZE	(PSION_LCD_W*PSION_LCD_H*4/8)	/* LCD buffer size */
#define LCD_MEM_SIZE	(LCD_PAL_SIZE + LCD_BUF_SIZE)	/* LCD memory size */
#define LCD_MEM_START	(PAGE_OFFSET + PSION_LCD_OFF)	/* LCD memory start offset */
#define LCD_PAL_START	LCD_MEM_START			/* LCD palette start offset */
#define LCD_BUF_START	(LCD_PAL_START + LCD_PAL_SIZE)	/* LCD buffer start offset */

#define LCDCTL		(0x0200)
#define LCDCTL_EN	(1 << 0)
#define LCDCTL_BW	(1 << 1)
#define LCDCTL_DP	(1 << 2)
#define LCDCTL_DONE	(1 << 3)
#define LCDCTL_NEXT	(1 << 4)
#define LCDCTL_ERR	(1 << 5)
#define LCDCTL_TFT	(1 << 7)
#define LCDCTL_M8B	(1 << 8)

#define LCDST		(0x0204)
#define LCDST_NEXT	(1 << 1)
#define LCDST_BER	(1 << 2)
#define LCDST_ABC	(1 << 3)
#define LCDST_FUF	(1 << 5)

#define LCD_DBAR1       (0x0210)

#define LCDT0		(0x0220)
#define LCDT1		(0x0224)
#define LCDT2		(0x0228)
#define LCDT2_IVS	(1 << 20)
#define LCDT2_IHS	(1 << 21)
#define LCDT2_IPC	(1 << 22)
#define LCDT2_IEO	(1 << 23)

#define LCD_DEF_CMASK	(0x10)		/* Default value for the LCD contrast */

/*
 * ETNA PCMCIA Controller
 */
#if defined(CONFIG_PSIONW_5MXPRO24MB) || defined(CONFIG_PSIONW_5MXPRO32MB)
#define ETNA_P_BASE	0x50000000	/* ETNA controller physical base */
#else
#define ETNA_P_BASE	0x20000000	/* ETNA controller physical base */
#endif

#define ETNA_V_BASE     0xde000000	/* ETNA controller virtual base */
#define ETNA_SIZE	0x00100000	/* ETNA controller size */

#define ETNA_UNKNOWN_0	0x0		/* Seems to be always 0x0f */
#define ETNA_UNKNONW_1	0x1		/* Seems to be always 0x00 */
#define ETNA_UNKNOWN_2	0x2		/* Seems to be always 0x02 */
#define ETNA_UNKNOWN_3	0x3		/* Seems to be always 0x00 */
#define ETNA_UNKNOWN_4	0x4		/* Seems to be always 0x00 */
#define ETNA_UNKNOWN_5	0x5		/* Seems to be always 0x00 */
#define ETNA_INT_STATUS	0x6		/* ETNA interrupt Status */
#define ETNA_INT_MASK	0x7		/* ETNA interrupt Mask */
#define ETNA_INT_CLEAR	0x8		/* ETNA interrupt Clear */
#define ETNA_SKT_STATUS	0x9		/* Socket status ??? */
#define ETNA_SKT_CFG	0xa		/* Socket config ??? */
#define ETNA_SKT_CTRL	0xb		/* Enable socket ??? */
#define ETNA_WAKE_1	0xc		/* Socket wake up; Usually 0x88, goes to 0x00 after STBY */
#define ETNA_SKT_ACTIVE	0xd		/* Socket active ??? */
#define ETNA_UNKNOWN_E	0xe		/* Seems to be always 0x00 */
#define ETNA_WAKE_2	0xf		/* Socket wake up; Usually 0x10, goes to 0x00 after STBY */

#define SKT_BUSY	(1 << 0)	/* ETNA socket busy */
#define SKT_NOT_READY	(1 << 1)	/* ETNA socket not ready */
#define SKT_CARD_OUT	(1 << 2)	/* ETNA card out */

#define ETNA_CARD_INT   (1 << 0)	/* ETNA card interrupt */
#define ETNA_BUSY_INT   (1 << 2)	/* ETNA socket interrupt */
#define ETNA_SOCK_INT1	(1 << 4)
#define ETNA_SOCK_INT2	(1 << 5)

#define CARD_INT_MASK	0x05		/* Normal card interrupt = ETNA_CARD_INT | ETNA_BUSY_INT */
#define ETNA_CLEAR_MASK	0xff		/* Clear ETNA interrupt */
#define	ETNA_CF_IRQ	0x01	        /* Enable ETNA Interrupt */

#if defined(CONFIG_PSIONW_5MXPRO24MB) || defined(CONFIG_PSIONW_5MXPRO32MB)
#define CF1_P_BASE	0x30000000	/* PCMCIA physical base */
#else
#define CF1_P_BASE	0x40000000	/* PCMCIA physical base */
#endif

#define CF1_V_BASE	0xe0000000	/* PCMCIA virtual base */
#define CF_SIZE		0x10000000	/* 4 areas of 64MB each */
#define CF_ATTR_BASE	0x00000000	/* This is needed, or it won't work!!! */
#define CF_MEM_BASE	0x04000000	/* Address bit 26 set */
#define CF_IO8_BASE	0x08000000	/* Address bit 27 set */
#define CF_IO16_BASE    0x0c000000	/* Address bits 26 & 27 set */

#define CF_CFG_REG	0x003c
#define CF_CFG_VAL	0x00730069	/* Epoc value from e 0x5801603c in ArLo */
#define CF_CUR_CFG     	0x200

/*
 * Boot EEPROM used by Epoc (Verified on 5mx Pro 32MB only)
 */
#define PSION_P_BR	0x00000000
#define PSION_V_BR	0xde200000
#define PSION_BR_SIZE	(1024*128)

/*
 * Boot Flash used by Epoc, not used in 5mx Pro (Not verified on 5mx)
 */
#define PSION_P_BF	0x10000000
#define PSION_V_BF	0xde300000
#define PSION_BF_SIZE	(1024*1024*2)

/*
 * GPIO/KBD Registers
 */
#define PADR		(0x0e00)
#define PBDR		(0x0e04)
#define PCDR		(0x0e08)
#define PDDR		(0x0e0c)
#define PADDR		(0x0e10)
#define PBDDR		(0x0e14)
#define PCDDR		(0x0e18)
#define PDDDR		(0x0e1c)
#define PEDR		(0x0e20)
#define PEDDR		(0x0e24)
#define KSCAN		(0x0e28)
#define LCDMUX		(0x0e2c)

/*
 * Port A, Keyboard
 */
#define PADR_0		(1 << 0)
#define PADR_1		(1 << 1)
#define PADR_2		(1 << 2)
#define PADR_3		(1 << 3)
#define PADR_4		(1 << 4)
#define PADR_5		(1 << 5)
#define PADR_6		(1 << 6)
#define PADR_7		(1 << 7)

/*
 * Port B, Data copied from Psion 5 code
 * 
 * +--7--+--6--+--5--+--4--+--3--+--2--+--1--+--0--+
 * |VPCEN|OPEN |VLDD3|VLDD2|VLDD1|VLDD0|EECLK|EECS |
 * | out |in/ou|in/ou|in/ou|in/ou|in/ou| out | out |
 * +-----+-----+-----+-----+-----+-----+-----+-----+
 */
#define PBDR_EECS	(1 << 0)	/* Ext clock?, NOT TESTED */
#define PBDR_EECLK	(1 << 1)	/* Ext clock?,  NOT TESTED */
#define PBDR_VLDD0	(1 << 2)	/* LCD contrast reg0 */
#define PBDR_VLDD1	(1 << 3)	/* LCD contrast reg1 */
#define PBDR_VLDD2	(1 << 4)	/* LCD contrast reg2 */
#define PBDR_VLDD3	(1 << 5)	/* LCD contrast reg3 */
#define PBDR_OPEN	(1 << 6)	/* Case open/closed */
#define PBDR_VPCEN	(1 << 7)	/* ETNA CF power disabled */

#define PBDR_VLD_MASK	0x3c		/* LCD contrast mask */
#define PBDR_VLD_SHIFT	2		/* LCD contrast shift */


/*
 * Port C, Data copied from Psion 5 code
 *
 * +--7--+--6--+--5--+--4--+--3--+--2--+--1--+--0--+
 * |ADICT|BBLD |UART1|LIGHT|UART2|PLED | DTR | RTS |
 * | out | out | out | out | out | out | out | out |
 * +-----+-----+-----+-----+-----+-----+-----+-----+
 */
#define PCDR_RTS	(1 << 0)	/* RS-232 RTS, NOT TESTED */
#define PCDR_DTR	(1 << 1)	/* RS-232 DTR toggle, NOT TESTED */
#define PCDR_PLED	(1 << 2)	/* Disable power LED, NOT TESTED */
#define PCDR_UART2	(1 << 3)	/* Enable UART1 */
#define PCDR_LIGHT	(1 << 4)	/* LCD backlight */
#define PCDR_UART1	(1 << 5)	/* Enable UART0 */
#define PCDR_ADICT	(1 << 6)	/* Set audio to dictaphone */

/*
 * Port D, Data copied from Psion 5 code
 *
 * +--7--+--6--+--5--+--4--+--3--+--2--+--1--+--0--+
 * | X2  |LCDLR|TRIX2|SLED |DORSW| LCD |AMPEN|CDEN |
 * |in/ou| out | n/a | out | out | out | out | out |
 * +-----+-----+-----+-----+-----+-----+-----+-----+
 */
#define PDDR_CDE	(1 << 0)	/* Codec enable, NOT TESTED */
#define PDDR_AMPEN	(1 << 1)	/* Audio amp enable, NOT TESTED */
#define PDDR_LCD_PWR	(1 << 2)	/* LCD power */
#define PDDR_ETNA_DOOR	(1 << 3)	/* ETNA door switch power, NOT TESTED */
#define PDDR_SLED	(1 << 4)	/* Front status LED on */
#define PDDR_PUMP_PWR2	(1 << 5)	/* ETNA and LCD need this */
#define PDDR_PUMP_PWR1	(1 << 6)	/* DC to DC converter power */
#define PDDR_ETNA_ERR  	(1 << 7)	/* ETNA access error */ 

/*
 * Port E, Data copied from Psion 5 code
 *
 * +--3--+--2--+--1--+--0--+
 * |Y1EN |X1EN |Y2EN |X2EN |
 * | out | out | out | out |
 * +-----+-----+-----+-----+
 */
#define PEDR_X2EN	(1 << 0)	/* Digitiser X2 drive, NOT TESTED */
#define PEDR_Y2EN	(1 << 1)	/* Digitiser Y2 drive, NOT TESTED */
#define PEDR_X1EN	(1 << 2)	/* Digitiser X1 drive, NOT TESTED */
#define PEDR_Y1EN	(1 << 3)	/* Digitiser Y1 drive, NOT TESTED */

#define MEMCFG1		(0x0000)	/* Memory configuration */
#define MEMCFG2		(0x0004)	/* Memory configuration */
#define DRAM_CFG	(0x0100)	/* DRAM control register */

/*
 * State and Power Controller
 */
#define PWRSR		(0x0400)	/* Power control state */
#define PWRCNT		(0x0404)	/* Clock/debug control status */
#define HALT		(0x0408) 	/* Enter idle mode */
#define STBY		(0x040c)	/* Enter standby mode */

#define BLEOI		(0x0410)	/* Clear battery low interrupt, write only */
#define MCEOI		(0x0414)	/* Clear media changed interrupt, write only */
#define TEOI		(0x0418)	/* Clear tick interrupt, write only */
#define STFCLR		(0x041c)	/* Clear NBFLG, RSTFLG, PFFLG, CLDFLG, write only */
#define E2EOI		(0x0420)	/* Clear NEINT2, write only */

#define TC1EOI		(0x0c0c)	/* Clear timer 1 interrupt, write only */
#define TC2EOI		(0x0c2c)	/* Clear timer 2 interrupt, write only */
#define RTCEOI		(0x0d10)	/* Clear RTC interrupt, write only */
#define UMSEOI		(0x0714)	/* Clear UART 2 modem status interrupt, write only */

/*
 * PWRSR Flags
 */
#define RTCDIV		(1 << 0)       	/* 6 bits, 64Hz ticks since the increment of RTC, 32 counts ahead of RTC */
#define MCDR		(1 << 6)	/* Media changed direct read */
#define DCDET		(1 << 7)	/* Power from the mains adapter */
#define WUDR		(1 << 8)	/* Wake up direct read */
#define WUON		(1 << 9)	/* Wake up out of stanby, clear at HALT STDBY or STFCLR */
#define NBFLG		(1 << 10)	/* New battery flag, clear at STFCLR */
#define RSTFLG		(1 << 11)	/* Reset flag, clear at STFCLR */
#define PFFLG		(1 << 12)	/* Power fail flag, clear at STFCLR */
#define CLDFLG		(1 << 13)	/* Cold start flag, clear at STFCLR */
#define VERID		(1 << 14)	/* Windermere version id, two bits, 0 for first version */

/*
 * PWRCNT Flags
 */
#define EXCKEN		(1 << 0)	/* External expansion clock enable, 1=on, 0=on only for bit 7 memcfg */
#define WAKEDIR		(1 << 1)	/* Disable waking up from STANDBY mode via the wakeup input */
#define CLKFLG		(1 << 2)	/* Clock speed, 1=36MHz, 0=18MHz */
#define ADCCLK		(1 << 3)	/* 8 bit to set the clock divider for ADCCLK output clock */

/*
 * Interrupts
 */
#define INTSR		(0x0500)	/* Interrupt status after masking, read-only */
#define INTRSR		(0x0504)	/* Interrupt status before masking, read-only */
#define INTENS		(0x0508)	/* Interrupt enable, read-write */
#define INTENC		(0x050c)	/* Interrupt disable, write-only */
#define INTTEST1	(0x0514)	/* Interrupt test register */
#define INTTEST2	(0x0518)	/* Interrupt test register */

#define INT_EXTFIQ	(1 << 0)
#define INT_BLINT	(1 << 1)
#define INT_WEINT	(1 << 2)
#define INT_MCINT	(1 << 3)
#define INT_CSINT	(1 << 4)
#define INT_EINT1	(1 << 5)	/* PCMCIA/external interrupt 1 */
#define INT_EINT2	(1 << 6)
#define INT_EINT3	(1 << 7)
#define INT_TC1OI	(1 << 8)
#define INT_TC2OI	(1 << 9)
#define INT_RTCMI	(1 << 10)
#define INT_TINT	(1 << 11)
#define INT_UART1	(1 << 12)
#define INT_UART2	(1 << 13)
#define INT_LCDINT	(1 << 14)
#define INT_SSEOTI	(1 << 15)

/*
 * Clock
 */
#define RTCTIME	((psionw_readl(RTCDRU) << 16) | (psionw_readl(RTCDRL) & 0xffff))

#define TC1LOAD		(0x0c00)
#define TC1VAL		(0x0c04)
#define TC1CTRL		(0x0c08)

#define TC2LOAD		(0x0c20)
#define TC2VAL		(0x0c24)
#define TC2CTRL		(0x0c28)
#define TC_BIT2		(1 << 2)
#define TC_CLKSEL	(1 << 3)
#define TC_BIT4		(1 << 4)
#define TC_BIT5		(1 << 5)
#define TC_MODE		(1 << 6)	
#define TC_ENABLE	(1 << 7)
#define BZCONT		(0x0c40)
#define BZ_BZTOG	(1 << 0)
#define BZ_BZMOD	(1 << 1)

#define RTCDRL		(0x0d00)	/* RTC data register low */
#define RTCDRU		(0x0d04)	/* RTC data register high */
#define RTCMRL		(0x0d08)	/* RTC match register low */
#define RTCMRU		(0x0d0c)	/* RTC match register high */

/*
 * SSI - Synchronous Serial Interface
 */
#define SSCR0		(0x0b00)	/* Serial control register 0 */
#define SSCR1		(0x0b04)	/* Serial control register 1 */
#define SSDR		(0x0b0c)	/* Data register */
#define SSSR		(0x0b14)	/* Status register */

/*
 * DC to DC Converter
 */
#define PUMPCON		(0x0900)
#define PUMP_RUN_VAL	(0xbbb)
#define PUMP_STOP_VAL	(0x0)

/*
 *  Codec
 */
#define CODR		(0x0a00)	/* Codec data register */
#define CONFG		(0x0a04)	/* Codec config register */
#define COLFG		(0x0a08)	/* Codec flag register */
#define COEOI		(0x0a0c)	/* Codec end of interrupt, write only */
#define COTEST		(0x0a10)	/* Codec test register */

#endif /* __ASM_HARDWARE_PSIONW_H */					\
